<?php
if (!isset($rootdir))
	die("");

// TOC functions. 
// Todo: provide indexing so the order of the listing can be controlled.
// For now, if you need a specific order, edit the TOC file by hand (or use a lside.html page) :)


///////////////////////////////////////////////////////////////////////
// Dummy TOC. Uses a directory listing as a TOC.
// The only advantage is you see all the pages in the pagedir
// directory. But there is no way effective way to store the title string.
// We create a separate file page.title to do this. Kluge. Nasty.
// I wouldn't use this unless you only had maybe a dozen or fewer pages.
///////////////////////////////////////////////////////////////////////
class tocBase
{
	function GetTOC()
	{
		global $pagedir;

		$names = array();
		if (is_dir($pagedir))
		{
			if (false !== ($dh = opendir ($pagedir)))
			{
				while (false !== ($id = readdir($dh)))
				{
					if ($id != "." && $id != "..")
					{
						$id = basename($id, ".html");
						if (file_exists("$pagedir/$id.title"))
							$names[$id] = file_get_contents("$pagedir/$id.title");
					}
				}
				closedir($dh);
			}
		}
		return $names;
	}

	function GetFormattedTOC()
	{
		$list = $this->GetTOC();

		echo "<ul>\n";
		foreach ($list as $itm => $title)
			echo "<li><a href=\"index.php?page=$itm\">$title</a></li>\n";
		echo "</ul>\n";
	}

	function UpdateTOC($page, $title)
	{
		global $pagedir;
		return file_put_contents("$pagedir/$page.title", $title);
	}

	function SetTOC($content)
	{
		return false;
	}

	function RemoveTOC($pgid)
	{
		global $pagedir;
		@unlink("$pagdir/$page.title");
		return true;
	}

	function PageTitle($pgid)
	{
		global $pagedir;
		if (file_exists("$pagedir/$page.title"))
			return file_get_contents("$pagedir/$page.title");
		return "";
	}

	function GetRSS()
	{
		global $pagedir;

		clearstatcache();
		$rssitems = array();
		$names = $this->GetTOC();
		foreach ($names as $title => $pgid)
		{
			$dt = filemtime("$pagedir/$pgid.html");
			$date = gmdate('r', $dt);
			if (($fh = fopen("$pagedir/$pgid.html", "r")))
			{
				$content = fread($fh, 80);
				fclose($fh);
			}
			else
				$content = $title;
			$rssitems[$dt] = array($pgid, $title, $date, $content, "");
			krsort($rssitems, SORT_NUMERIC);
		}

		return $rssitems;
	}
}

///////////////////////////////////////////////////////////////////////
// XML based TOC. The XML structure is:
// <toc>
//   <item id="pageid" title="title" />
// </toc>
// This works well even with a large number of pages. It is
// easy to expand to include other information about the page by
// adding additional atributes to the <item> nodes.
///////////////////////////////////////////////////////////////////////
class tocXML extends tocBase
{
	var $doc;
	var $root;
	var $tocfile;

	function tocXML()
	{
		global $pagedir;

		$this->doc = NULL;
		$this->root = NULL;

		$this->tocfile = "$pagedir/toc.xml";
		if (file_exists($this->tocfile))
		{
			$this->doc = domxml_open_file($this->tocfile);
			if ($this->doc)
				$this->root = $this->doc->document_element();
		}
	}

	function GetTOC()
	{
		global $pagedir;

		$names = array();

		if ($this->doc && $this->root)
		{
			$pages = $this->doc->get_elements_by_tagname("item");
			foreach ($pages as $page)
			{
				$id = $page->get_attribute("id");
				$label = $page->get_attribute("title");
				$names[$id] = $label;
			}
			unset($pages);
		}

		return $names;
	}

	function UpdateTOC($pgid, $title)
	{
		global $theUser;

		$pagenode = $this->FindPage($pgid);
		if (!$pagenode)
		{
			$pagenode = $this->doc->create_element("item");
			$pagenode = $this->root->append_child($pagenode);
			$pagenode->set_attribute("id", $pgid);
			$pagenode->set_attribute("slug", 120);
		}
		$pagenode->set_attribute("title", $title);
		$pagenode->set_attribute("dt", time());
		$pagenode->set_attribute("auth", $theUser->loginid);
		unset($pagenode);
		$xmlstr = $this->doc->dump_mem(1, "ISO-8859-1");
		return file_put_contents($this->tocfile, $xmlstr);
		// the dump function will not preserve formatting, no matter what the manual says...
		//return $this->doc->dump_file($this->tocfile, false, true);
	}

	function RemoveTOC($pgid)
	{
		$pagenode = $this->FindPage($pgid);
		if ($pagenode)
		{
			$this->root->remove_child($pagenode);
			unset($pagenode);
			$xmlstr = $this->doc->dump_mem("ISO-8859-1", 1);
			return file_put_contents($this->tocfile, $xmlstr);
		}
		return false;
	}

	function SetTOC($content)
	{
		if ($content == "")
			return false;

		$dom = domxml_open_mem($content);
		if ($dom)
		{
			unset($this->root);
			unset($this->dom);
			$this->doc = $dom;
			$this->root = $this->doc->document_element();
			$bak = $this->tocfile.".bak";
			@unlink($bak);
			rename($this->tocfile, $bak);
			$xmlstr = $this->doc->dump_mem(1, "ISO-8859-1");
			return file_put_contents($this->tocfile, $xmlstr);
		}
		return false;
	}

	function PageTitle($pgid)
	{
		$pagenode = $this->FindPage($pgid);
		if ($pagenode)
			return $pagenode->get_attribute("title");
		return "";
	}

	function GetRSS()
	{
		global $pagedir;

		clearstatcache();

		$rssitems = array();

		if ($this->doc && $this->root)
		{
			$pages = $this->doc->get_elements_by_tagname("item");
			foreach ($pages as $page)
			{
				$pgid = $page->get_attribute("id");
				$title = $page->get_attribute("title");
				if ($page->has_attribute("auth"))
					$auth = $page->get_attribute("auth");
				else
					$auth = "";
				if ($page->has_attribute("slug"))
					$slug = $page->get_attribute("slug");
				else
					$slug = 100;
				if ($page->has_attribute("dt"))
					$dt = $page->get_attribute("dt");
				else
					$dt = filemtime("$pagedir/$pgid.html");
				$date = gmdate('r', $dt);
				if (($fh = fopen("$pagedir/$pgid.html", "r")))
				{
					$content = fread($fh, 100);
					fclose($fh);
				}
				else
					$content = $title;
				$rssitems[$dt] = array($pgid, $title, $date, $content, $auth);
			}
			unset($pages);
			krsort($rssitems, SORT_NUMERIC);
		}

		return $rssitems;
	}

	function FindPage($pgid)
	{
		if ($this->doc && $this->root)
		{
			$pages = $this->doc->get_elements_by_tagname("item");
			foreach ($pages as $pagenode)
			{
				if ($pgid == $pagenode->get_attribute("id"))
					return $pagenode;
			}
			unset($pagenode);
			unset($pages);
		}

		return NULL;
	}
}

///////////////////////////////////////////////////////////////////////
// Text file based TOC. Each line in the file is of the form
// pageid=title
// This should only be used with a small number of pages and it is
// not easy to expand to include other information about the file.
// However, the TOC file is easy to edit by hand if desired.
///////////////////////////////////////////////////////////////////////

class tocTXT extends tocBase
{
	var $items;
	var $tocfile;

	function tocTXT()
	{
		global $pagedir;

		$this->items = array();
		$this->tocfile = "$pagedir/toc.txt";

		// toc.txt is of the form page=label\npage2=label2\n...
		if (file_exists($this->tocfile))
		{
			$toc = file_get_contents($this->tocfile);
			$this->items = explode("\n", $toc);
		}
	}

	function GetTOC()
	{
		$names = array();
		foreach ($this->items as $itm)
		{
			list($id, $title) = explode("=", $itm);
			$names[$id] = $title;
		}
		return $names;
	}

	function UpdateTOC($pgid, $title)
	{
		if (($ndx = $this->FindPage($pgid)) !== false)
		{
			$this->items[$ndx] = "$pgid=$title";
		}
		else
		{
			$this->items[] = "$pgid=$title";
		}
		return file_put_contents($this->tocfile, implode("\n", $this->items));
	}

	function SetTOC($content)
	{
		@unlink($this->tocfile.".bak");
		rename($this->tocfile, $this->tocfile.".bak");
		$ok = file_put_contents($this->tocfile, $content);
		$this->items = explode("\n", $content);
		return $ok;
	}

	function RemoveTOC($pgid)
	{
		if (($ndx = $this->FindPage($pgid)) !== false)
		{
			unset($this->items[$ndx]);
			return file_put_contents($this->tocfile, implode("\n", $this->items));
		}
		return true;
	}

	function PageTitle($pgid)
	{
		if (($ndx = $this->FindPage($pgid)) !== false)
		{
			list($itm, $title) = explode("=", $this->items[$ndx]);
			return $title;
		}
		return "";
	}

	function GetRSS()
	{
		global $pagedir;

		clearstatcache();

		$rssitems = array();

		foreach ($this->items as $itm)
		{
			list($pgid, $title) = explode("=", $itm);
			$dt = filemtime("$pagedir/$pgid.html");
			$date = gmdate('r', $dt);
			if (($fh = fopen("$pagedir/$pgid.html", "r")))
			{
				$content = fread($fh, 80);
				fclose($fh);
			}
			else
				$content = $title;
			$rssitems[$dt] = array($pgid, $title, $date, $content, "");
			krsort($rssitems, SORT_NUMERIC);
		}

		return $rssitems;
	}

	function FindPage($pgid)
	{
		$count = count($this->items);
		for ($ndx = 0; $ndx < $count; $ndx++)
		{
			$itm = explode("=", $this->items[$ndx]);
			if ($itm[0] == $pgid)
				return $ndx;
		}
		return false;
	}
}

?>